/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.parser.output;

import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.logger.Logger;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import net.sourceforge.chaperon.parser.ParserTable;

/**
 * This class represents a adapter from the parser events to SAX events
 *
 * @author Stephan Michels
 * @version CVS $Id: SAXEventAdapter.java,v 1.4 2002/05/08 15:34:12 benedikta Exp $
 */
public class SAXEventAdapter implements EventListener, LogEnabled
{
  private ContentHandler _handler = null;
  private boolean _embedded = false;
  private boolean _includeIgnorableTokens = false;
  private Logger _logger;

  public SAXEventAdapter(ContentHandler handler)
  {
    if (handler==null)
      throw new NullPointerException();

    _handler = handler;
  }

  public SAXEventAdapter(ContentHandler handler, boolean includeIgnorableTokens, boolean embedded)
  {
    if (handler==null)
      throw new NullPointerException();

    _handler = handler;
    _includeIgnorableTokens = includeIgnorableTokens;
    _embedded = embedded;
  }

  /**
   * Provide component with a logger.
   *
   * @param logger the logger
   */
  public void enableLogging(Logger logger)
  {
    _logger = logger;
  }

  public void startDocument(ParserTable table)
  {
    try
    {
      if (!_embedded)
        _handler.startDocument();
      _handler.startPrefixMapping("cpn", table.getURI());
    } catch (SAXException e)
    {
      if (_logger!=null)
        _logger.error("Could not fire SAX event", e);
    }
  }

  public void startProduction(ParserTable table, int productionindex)
  {
    try
    {
      String name = table.getNonTerminalSymbol(table.getProductionSymbol(productionindex));
      _handler.startElement(table.getURI(), name, "cpn:"+name, new AttributesImpl());
    } catch (SAXException e)
    {
      if (_logger!=null)
        _logger.error("Could not fire SAX event", e);
    }
  }

  public void startToken(ParserTable table, int tokenindex, char[] text, int textstart, int textlength)
  {
    try
    {
      String name = table.getTerminalSymbol(tokenindex);
      _handler.startElement(table.getURI(), name, "cpn:"+name, new AttributesImpl());

      _handler.characters(text, textstart, textlength);
    } catch (SAXException e)
    {
      if (_logger!=null)
        _logger.error("Could not fire SAX event", e);
    }
  }

  public void endToken(ParserTable table, int tokenindex)
  {
    try
    {
      String name = table.getTerminalSymbol(tokenindex);
      _handler.endElement(table.getURI(), name, "cpn:"+name);
    } catch (SAXException e)
    {
      if (_logger!=null)
        _logger.error("Could not fire SAX event", e);
    }
  }

  public void startIgnorableToken(ParserTable table, int ignorabletokenindex, 
                                  char[] text, int textstart, int textlength)
  {
    if (_includeIgnorableTokens)
    { 
      try
      {
        String name = table.getIgnorableTokenSymbol(ignorabletokenindex);
        _handler.startElement(table.getURI(), name, "cpn:"+name, new AttributesImpl());

        _handler.characters(text, textstart, textlength);
      } catch (SAXException e)
      {
        if (_logger!=null)
          _logger.error("Could not fire SAX event", e);
      }
    }
  }

  public void endIgnorableToken(ParserTable table, int ignorabletokenindex)
  {
    if (_includeIgnorableTokens)
    {
      try
      {
        String name = table.getIgnorableTokenSymbol(ignorabletokenindex);
        _handler.endElement(table.getURI(), name, "cpn:"+name);
      } catch (SAXException e)
      {
        if (_logger!=null)
          _logger.error("Could not fire SAX event", e);
      }
    }
  }

  public void endProduction(ParserTable table, int productionindex)
  {
    try
    {
      String name = table.getNonTerminalSymbol(table.getProductionSymbol(productionindex));
      _handler.endElement(table.getURI(), name, "cpn:"+name);
    } catch (SAXException e)
    {
      if (_logger!=null)
        _logger.error("Could not fire SAX event", e);
    }
  }

  public void endDocument(ParserTable table)
  {
    try
    {
      _handler.endPrefixMapping("cpn");

      if (!_embedded)
        _handler.endDocument();
    } catch (SAXException e)
    {
      if (_logger!=null)
        _logger.error("Could not fire SAX event", e);
    }
  }
}
